/***
* MIT License
*
* Copyright (c) 2016 Morten Bojsen-Hansen
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
***/

#ifndef _SIM_PMLCOMPUTEDIVERGENCE_H
#define _SIM_PMLCOMPUTEDIVERGENCE_H

#include <GAS/GAS_SubSolver.h>

#define PML_SPLIT_DENSITY "splitdensity"
#define PML_LAYER_THICKNESS "layerthickness"
#define PML_MAXIMUM_DAMPING "maximumdamping"
#define PML_DAMPING_EXPONENT "dampingexponent"
#define PML_DAMPING "damping"
#define PML_DAMP_PLUS_X "dampplusx"
#define PML_DAMP_PLUS_Y "dampplusy"
#define PML_DAMP_PLUS_Z "dampplusz"
#define PML_DAMP_MINUS_X "dampminusx"
#define PML_DAMP_MINUS_Y "dampminusy"
#define PML_DAMP_MINUS_Z "dampminusz"

class SIM_PMLComputeDivergence : public GAS_SubSolver
{
public:
	SIM_PMLComputeDivergence(const SIM_DataFactory* factory);
	virtual ~SIM_PMLComputeDivergence();
	bool solveGasSubclass(SIM_Engine &engine, SIM_Object *obj, SIM_Time time, SIM_Time timestep) override;

	GET_DATA_FUNC_I(PML_LAYER_THICKNESS, LayerThickness);
	GET_DATA_FUNC_F(PML_MAXIMUM_DAMPING, MaximumDamping);
	GET_DATA_FUNC_F(PML_DAMPING_EXPONENT, DampingExponent);
	GET_DATA_FUNC_B(PML_DAMPING, Damping);
	GET_DATA_FUNC_B(PML_DAMP_PLUS_X, DampPlusX);
	GET_DATA_FUNC_B(PML_DAMP_PLUS_Y, DampPlusY);
	GET_DATA_FUNC_B(PML_DAMP_PLUS_Z, DampPlusZ);
	GET_DATA_FUNC_B(PML_DAMP_MINUS_X, DampMinusX);
	GET_DATA_FUNC_B(PML_DAMP_MINUS_Y, DampMinusY);
	GET_DATA_FUNC_B(PML_DAMP_MINUS_Z, DampMinusZ);

private:
	double sigma(double x);

	bool shouldMultiThread(const SIM_RawField *field) const { return field->field()->numTiles() > 1; }

	// copy+pasta from SIM_PMLContract
	THREADED_METHOD2(SIM_PMLComputeDivergence,
		shouldMultiThread(dst),
		contract,
		SIM_RawField*, dst,
		const SIM_RawField**, src);
	void contractPartial(
		SIM_RawField *dst,
		const SIM_RawField* src[3],
		const UT_JobInfo &info);

	static const SIM_DopDescription* getDopDescription();
	DECLARE_STANDARD_GETCASTTOTYPE();
		DECLARE_DATAFACTORY(
		SIM_PMLComputeDivergence,
		GAS_SubSolver,
		"PML Compute Divergence", // DOP name (mirror in cpp)
		getDopDescription());
};

#endif // _SIM_PMLCOMPUTEDIVERGENCE_H
